@@ -24,6 +24,10 @@ import itertools
 import threading
 import time
 
+try:
+    import eventlet
+except ImportError:
+    eventlet = None
 import memcache
 from oslo_log import log
 from six.moves import queue
@@ -45,9 +49,17 @@ class _MemcacheClient(memcache.Client):
     """
     __delattr__ = object.__delattr__
     __getattribute__ = object.__getattribute__
-    __new__ = object.__new__
     __setattr__ = object.__setattr__
 
+    # Hack for lp 1812935
+    if eventlet and eventlet.patcher.is_monkey_patched('thread'):
+        # NOTE(bnemec): I'm not entirely sure why this works in a
+        # monkey-patched environment and not with vanilla stdlib, but it does.
+        def __new__(cls, *args, **kwargs):
+            return object.__new__(cls)
+    else:
+        __new__ = object.__new__
+
     def __del__(self):
         pass
 
@@ -123,6 +135,7 @@ class ConnectionPool(queue.Queue):
     @contextlib.contextmanager
     def acquire(self):
         self._trace_logger('Acquiring connection')
+        self._drop_expired_connections()
         try:
             conn = self.get(timeout=self._connection_get_timeout)
         except queue.Empty:
@@ -135,7 +148,6 @@ class ConnectionPool(queue.Queue):
             yield conn
         finally:
             self._trace_logger('Releasing connection %s', id(conn))
-            self._drop_expired_connections()
             try:
                 # super() cannot be used here because Queue in stdlib is an
                 # old-style class
